package com.fzm.common.mq;

import com.alibaba.fastjson.JSON;
import com.fzm.common.entity.Notify;
import com.fzm.common.entity.NotifyState;
import com.fzm.common.mapper.NotifyMapper;
import com.fzm.common.mapper.NotifyStateMapper;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author wt
 * @date 2022/3/7
 */
@Component
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class NotifyConsumer {
    private final NotifyMapper notifyMapper;
    private final NotifyStateMapper notifyStateMapper;


    @RabbitListener(queues = {NotifyConfig.NOTIFY_QUEUE})
    public void notifyMessageUser(String content) {
        if (StringUtils.isBlank(content)) {
            return;
        }
        Notify notify = JSON.parseObject(content, Notify.class);

        notifyMapper.insert(notify);

        Integer uid = notify.getUid();
        NotifyState notifyState = notifyStateMapper.selectByUid(uid);


        boolean doNotExist = notifyState == null;

        notifyState = doNotExist ? new NotifyState() : notifyState;
        notifyState.setUid(uid);

        // 获取用户原始消息通知id数组
        String nids = notifyState.getNids();
        nids = StringUtils.isBlank(nids) ? "[]" : nids;
        List<Integer> nidArray = JSON.parseArray(nids, Integer.class);

        // 最多500未读消息
        if (nidArray.size() >= 500) {
            nidArray.remove(0);
        }
        nidArray.add(notify.getId());
        nidArray.sort(Integer::compareTo);

        notifyState.setNumber(nidArray.size());
        notifyState.setNids(JSON.toJSONString(nidArray));

        if (doNotExist) {
            try {
                notifyStateMapper.insert(notifyState);
                return;
            } catch (Exception ignored) {
            }
        }
        notifyStateMapper.update(notifyState);
    }
}
