package com.fzm.portal.controller;

import com.alibaba.fastjson.JSON;
import com.fzm.common.annotation.Authentication;
import com.fzm.common.entity.NotifyState;
import com.fzm.common.entity.vo.NotifyVO;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.interceptor.LoginUserInfo;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.service.NotifyService;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import lombok.val;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author wt
 * @date 2022/3/7
 */
@Api(value = "通知", tags = "通知", description = " ")
@RestController
@Authentication
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@RequestMapping(value = "/notify", produces = MediaType.APPLICATION_JSON_VALUE)
public class NotifyController {
    private final NotifyService notifyService;

    @GetMapping("/pages")
    @ApiOperation(value = "分页查询")
    public ResponseModel<PageInfo<NotifyVO>> pages(@ApiParam(value = "页码", required = true) @RequestParam Integer pageNum,
                                                   @ApiParam(value = "每页记录数", required = true) @RequestParam Integer pageSize) {
        Integer userId = LoginUserInfo.getLoginUser().getUserId();
        PageInfo<NotifyVO> pageInfo = notifyService.pages(userId, pageNum, pageSize);
        return ResponseModel.success(pageInfo);
    }

    @ApiOperation("未读消息数量")
    @GetMapping("/count")
    public ResponseModel<NotifyState> messageCount() {
        Integer userId = LoginUserInfo.getLoginUser().getUserId();
        NotifyState notifyState = notifyService.selectByUid(userId);
        return ResponseModel.success(notifyState);
    }

    @ApiOperation("查询详细")
    @GetMapping("/detail")
    public ResponseModel<NotifyVO> getById(@RequestParam Integer id) {
        if (id == null || id <= 0) {
            throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
        }

        NotifyVO vo = notifyService.selectById(id);

        if (vo != null) {
            Integer userId = LoginUserInfo.getLoginUser().getUserId();
            NotifyState notifyState = notifyService.selectByUid(userId);
            if (notifyState.getNumber() != 0) {
                val nids = notifyState.getNids();
                val nidArray = JSON.parseArray(nids, Integer.class);
                if (nidArray.contains(id)) {
                    nidArray.remove(id);
                    notifyState.setNumber(nidArray.size());
                    notifyState.setNids(JSON.toJSONString(nidArray));
                    notifyService.updateState(notifyState);
                }
            }
        }

        return ResponseModel.success(vo);
    }

    @ApiOperation("全部已读")
    @GetMapping("/all-read")
    public ResponseModel<Boolean> allRead() {
        Integer userId = LoginUserInfo.getLoginUser().getUserId();
        NotifyState notifyState = notifyService.selectByUid(userId);
        if (notifyState.getNumber() != 0) {
            notifyState.setNumber(0);
            notifyState.setNids("[]");
            notifyService.updateState(notifyState);
        }

        return ResponseModel.success();
    }
}
