package com.fzm.portal.listener;

import cn.fzm.chain.simplesdk.client.ParaChainClient;
import cn.fzm.chain.simplesdk.constant.TxStatusEnum;
import cn.fzm.chain.simplesdk.model.TxResult;
import com.fzm.common.constant.SystemConstant;
import com.fzm.common.entity.Nft;
import com.fzm.common.entity.Order;
import com.fzm.common.entity.User;
import com.fzm.common.entity.dto.NftPublishMsg;
import com.fzm.common.enums.*;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.mq.NotifyPublisher;
import com.fzm.common.service.NftService;
import com.fzm.common.service.OrderService;
import com.fzm.common.service.UserService;
import com.fzm.common.service.WxPayService;
import com.fzm.common.utils.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author tangtuo
 * @date 2022/1/21 16:49
 */
@Slf4j
@Component
public class NftListener {

    @Resource
    private ParaChainClient paraChainClient;

    @Resource
    private NftService nftService;

    @Resource
    UserService userService;

    @Resource
    private RedisUtil redisUtil;

    @Resource
    private OrderService orderService;

    @Resource
    private WxPayService wxPayService;

    @Resource
    private NotifyPublisher notifyPublisher;

    /**
     * 监听nft发行结果
     *
     * @param msg
     */
    @RabbitListener(queues = "nft.publish.queue")
    public void listenNftPublish(NftPublishMsg msg) throws Exception {
        if (log.isDebugEnabled()) {
            log.debug("收到确认nft发行结果的消息: {}", msg);
        }

        Nft nft = nftService.getById(msg.getId());
        User user = userService.getUserByWallet(nft.getPublishAddress());

        try {
            String hash = msg.getHash();
            // 确认交易结果
            TxResult txResult = paraChainClient.cycleConfirmTxWithHash(hash, true, 1000);
            if (!TxStatusEnum.SUCCESS.equals(txResult.getStatus())) {
                throw GlobalException.newException(ResultCode.PUBLISH_ERROR, txResult.getErrMsg().getValue());
            }
            String realHash = paraChainClient.getRealTxHashFromGrp(hash);
            if (StringUtils.isBlank(realHash)) {
                throw GlobalException.newException(ResultCode.PUBLISH_ERROR);
            }
            nft.setNftHash(realHash);
            nft.setTokenId(msg.getTokenId());
            nft.setPublishStatus(PublishStatus.SUSSED.getCode());
            nftService.updateById(nft);
            // 如果用户是第一次发行作品,把用户的isPublish修改成1,并清空用户统计的缓存信息
            if (SystemConstant.BOOLEAN_DATA_FALSE.equals(user.getIsPublish())) {
                User u = new User().setId(user.getId()).setIsPublish(SystemConstant.BOOLEAN_DATA_TRUE);
                userService.updateById(u);
                redisUtil.delete("user::statistic");
            }

            notifyPublisher.nftSuccessNotify(user.getId(), nft.getName());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            // nft发行失败,需要把nft的发行状态改成failed,然后主动发起退款
            nft.setPublishStatus(PublishStatus.FAILED.getCode());
            nftService.updateById(nft);
            Order order = orderService.getByPaySceneAndProductId(PayScene.NFT.getCode(), nft.getId(), OrderStatus.PAYED.getStatus());
            if (order != null) {
                wxPayService.refund(order.getId(), RefundLaunchChannel.USER.getCode());
            }

            notifyPublisher.nftFailNotify(user.getId(), nft.getName(), e.getMessage());
        }
    }
}
